<?php
/**
* @version		1.6.2
* @package		Kunena Converters
* @subpackage	Kunena Converters
* @copyright	2009-2010 JoomAce LLC, www.joomace.net
* @copyright    2010 Hoffi (some modifications)
* @license		GNU/GPL http://www.gnu.org/copyleft/gpl.html

* Some functions are based on Agora (jvitals.com) component's coverter.
* This version converts from Agora to Kunena 1.6 (#__kunena tables)

* 1.6.1: Fixed problem with undefined users (id 0)
* 1.6.2: Fixed subscriptions bug (wrong id stored)
*/

//No Permision
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.application.component.model');

define('QUERY_LIMIT', 100);

/* --- SETTINGS -------------------------------------------------------------------------------- */
define('NO_GUEST_ACCESS', 0); /* Restrict access to all categories and forums to registered users.
                               * (can be changed later in Kunena Control Panel)
                               */

define('IMPORT_SMILEYS_MODE', 0); /* Import smileys from Agora:
                                   *  0 - no import
                                   *  1 - add new but don't replace files
                                   *  2 - add new and replace files (original files too)
                                   *  3 - clear table and import all smileys incl. replacing files
                                   */
/* --- End of SETTINGS ------------------------------------------------------------------------- */

class KunenaConvertersModelKunenaConverters extends JModel {
	
	protected static $new_forums_ids = array();
/*	protected static $new_parents_ids = array();  not used */
	
	// Main constructer
	function __construct() {
		parent::__construct();
	}
	
	public function agora() {
		/* TODO: Remove avatars (because we delete all kunena users) */
		self::_truncateTables();
		self::_importAgoraUsers();
		self::_importAgoraForums();
		/* Hoffi: stores id of first post per topic; set in _importAgoraTopics(), used in _importAgoraPosts() */
		$this->firstPost = array();
		self::_importAgoraTopics(); /* this MUST be called before _importAgoraPosts() */
		self::_importAgoraPosts();
		self::_importAgoraPolls();  /* TODO */
		self::_importAgoraSubscriptions();
		if (IMPORT_SMILEYS_MODE > 0) {
			self::_importAgoraSmilies();
		}
		self::_updateLastMessageIDofCats();
	}
	
	/* Clear all user and content related tables. (keeps general configuration untouched)
	 */
	protected function _truncateTables() {
		/* Because we clear all users, categories, topics, polls, and posts
		 * we should also clear all other tables referencing to these.
		 */
		$truncates = array();
		$truncates[] = 'TRUNCATE TABLE #__kunena_categories';
		$truncates[] = 'TRUNCATE TABLE #__kunena_favorites';
		$truncates[] = 'TRUNCATE TABLE #__kunena_messages';
		$truncates[] = 'TRUNCATE TABLE #__kunena_messages_text';
		$truncates[] = 'TRUNCATE TABLE #__kunena_moderation';
		$truncates[] = 'TRUNCATE TABLE #__kunena_polls';
		$truncates[] = 'TRUNCATE TABLE #__kunena_polls_options';
		$truncates[] = 'TRUNCATE TABLE #__kunena_polls_users';
		$truncates[] = 'TRUNCATE TABLE #__kunena_sessions';
		$truncates[] = 'TRUNCATE TABLE #__kunena_subscriptions';
		$truncates[] = 'TRUNCATE TABLE #__kunena_subscriptions_categories';
		$truncates[] = 'TRUNCATE TABLE #__kunena_thankyou';
		$truncates[] = 'TRUNCATE TABLE #__kunena_users';
		$truncates[] = 'TRUNCATE TABLE #__kunena_users_banned';
		if (IMPORT_SMILEYS_MODE == 3) {
			$truncates[] = 'TRUNCATE TABLE #__kunena_smileys';      /* This will remove all kunena smileys! */
		}
		@reset($truncates);
		
		while(list( ,$sql) = @each($truncates)) {
			$this->_db->setQuery($sql);
			$this->_db->query();
		}
	}
	
	/* Import user data from #__agora_users to #__kunena_users.
	 * Also copy used avatar files. (jpg, gif, png supported)
	 */
	protected function _importAgoraUsers() {
		$offset = 0;
		
		do {
			$have_data = false;
			
			$this->_db->setQuery("SELECT * FROM #__agora_users ORDER BY jos_id LIMIT {$offset}, " . QUERY_LIMIT);
			$ob_list = $this->_db->loadAssocList();
			
			if (!empty($ob_list)) {
				$have_data = true;
			} else {
				break;
			}
			
			foreach ($ob_list as $ob) {

				/* respect users decision not to show the birthday */
				/* Note: My misinterpretation; this flag really means "don't show year, but show day and month" */
				/*       So if someone implements conversion please add "0001-%m-%d" in case flag is set ;) */
				$birthdate = '';
				if (!$ob['hide_age'] > 0) {
					/* TODO: We must convert - but how? */
					/*$birthdate = strftime("%Y-%m-%d", $ob['birthdate']);*/
				}

				// Data array
				$data = array(
					'userid' => $ob['jos_id'],
					'posts' => $ob['num_posts'],
					'signature'	=> $ob['signature'],
					'birthdate' => $birthdate,
					'gender' => $ob['gender'],
					'location' => $ob['location'],
					'ICQ' => $ob['icq'],
					'AIM' => $ob['aim'],
					'MSN' => $ob['msn'],
					'SKYPE' => $ob['skype'],
					'websiteurl' => $ob['url']
				);

				/* Copy avatar */
				$dir_ = dirname(dirname(dirname(dirname(dirname(__FILE__)))));
				$members_av = $dir_.DS.'components'.DS.'com_agora'.DS.'img'.DS.'members'.DS.$ob['jos_id'].DS.$ob['jos_id'];
				$pre_av = $dir_.DS.'components'.DS.'com_agora'.DS.'img'.DS.'pre_avatars'.DS.$ob['id'];
				
				foreach (array('png', 'jpg', 'gif') as $ext) {
					if (JFile::exists($members_av.'.'.$ext)) {
						JFile::copy($members_av.'.'.$ext, $dir_.DS.'media'.DS.'kunena'.DS.'avatars'.DS.'users'.DS.'avatar'.$ob['jos_id'].'.'.$ext);
						$data['avatar'] = 'users/avatar'.$ob['jos_id'].'.'.$ext;
						break; /*foreach*/
					} elseif (JFile::exists($pre_av.'.'.$ext)) {
						JFile::copy($pre_av.'.'.$ext, $dir_.DS.'media'.DS.'kunena'.DS.'avatars'.DS.'users'.DS.'avatar'.$ob['jos_id'].'.'.$ext);
						$data['avatar'] = 'users/avatar'.$ob['jos_id'].'.'.$ext;
						break; /*foreach*/
					}
				}

				self::_insertData('kunena_users', $data);
			}
			
			$offset += QUERY_LIMIT;
			
		} while ($have_data);
	}
	
	/* Import categories from #__agora_categories to #__kunena_categories.
	 * Import forums from #__agora_forums to #__kunena_categories too.
	 * Restrict access to all registered users if "Administrator" group is found in #__core_acl_aro_groups)
	 */
	protected function _importAgoraForums() {
		$pubacc = 0;
		$admacc = 0;

		if (NO_GUEST_ACCESS > 0) {
			/* If we find Joomla Administrator group id we can restrict forums to registered users only. */
			$this->_db->setQuery("SELECT id FROM #__core_acl_aro_groups WHERE name = 'Administrator'");
			$adm_list = $this->_db->loadAssocList();
			if (count($adm_list) == 1) {
				$gid = $adm_list[0]['id'];
				if ($gid && ($gid != '0')) {
					$pubacc = '-1';  /* -1 = COM_KUNENA_ALLREGISTERED */
					$admacc = $gid;
				}
			}
		}
		
		$this->_db->setQuery("SELECT * FROM #__agora_categories ORDER BY id");
		$ob_list = $this->_db->loadAssocList();
		
		if (empty($ob_list)) {
			return;
		}
		
		foreach ($ob_list as $ob) {
			$data = array(
				'id' =>	$ob['id'],
				'name' => $ob['cat_name'],
				'published' => $ob['enable'],
				'parent' => '0',
				'ordering' => $ob['disp_position'],
				'pub_access' => $pubacc,
				'pub_recurse' => '1',
				'admin_access' => $admacc,
				'admin_recurse' => '1',
				'allow_polls' => '1'        /* new in kunena 1.6 */
			);
			
			self::_insertData('kunena_categories', $data);
		}
		
		$this->_db->setQuery("SELECT * FROM #__agora_forums ORDER BY parent_forum_id, cat_id, id");
		$ob_list = $this->_db->loadAssocList();
		
		if (empty($ob_list)) {
			return;
		}
		
		foreach ($ob_list as $ob) {
			if ($ob['parent_forum_id'] == 0) {
				$parent = $ob['cat_id'];
			}
			elseif (isset(self::$new_forums_ids[$ob['parent_forum_id']])) {
				$parent = self::$new_forums_ids[$ob['parent_forum_id']];
			}
			else {
				continue;
			}
			
			// Data array
			$data = array(
				'name' => $ob['forum_name'],
				'description' => $ob['forum_desc'],
				'published' => $ob['enable'],
				'moderators' => $ob['moderators'],
				'parent' => $parent,
				'ordering' => $ob['disp_position'],
				'numTopics' => $ob['num_topics'],
				'numPosts' => $ob['num_posts'],
				'id_last_msg' => $ob['last_post_id'],
				'time_last_msg' => $ob['last_post'],
				'locked' => $ob['closed'],
				'pub_access' => $pubacc,
				'pub_recurse' => '1',
				'admin_access' => $admacc,
				'admin_recurse' => '1'
			);
			
			$id = self::_insertdata('kunena_categories', $data);
			self::$new_forums_ids[$ob['id']] = $id;
		}	    			    
	}
	
	/* Import topics with first posts from #__agora_topics and #__agora_posts
	 * with help of #__agora_users to #__kunena_messages and #__kunena_messages_text.
	 * Map topic icons as good as possible.
	 */
	protected function _importAgoraTopics() {	
		$offset = 0;
		
		do {
			$have_data = false;
			
			$fields = "id, subject, closed, sticky, moved_to, forum_id, num_views, icon_topic";
			$this->_db->setQuery("SELECT {$fields} FROM #__agora_topics ORDER BY id LIMIT {$offset}, " . QUERY_LIMIT);
			$topic_list = $this->_db->loadAssocList();

			if (!empty($topic_list)) {
				$have_data = true;
			} else {
				break;
			}

			foreach ($topic_list as $topic) {
			/* Because there could be anonymous posts this query doesn't work. We need a more advanced one.
				$fields = "p.id, p.poster, p.poster_id, p.poster_ip, p.poster_email, p.message, p.posted, p.edited, p.edited_by, u.jos_id, u.email";
				$this->_db->setQuery("SELECT {$fields} FROM #__agora_posts AS p, #__agora_users AS u WHERE p.topic_id = {$topic['id']} AND p.poster_id = u.id ORDER BY p.id ASC LIMIT 1");
			 */
				$fields = "p.id, p.poster, p.poster_id, p.poster_ip, p.poster_email, p.message, p.posted, p.edited, p.edited_by";
				$this->_db->setQuery("SELECT {$fields}, u.jos_id, u.email FROM #__agora_posts AS p, #__agora_users AS u WHERE p.topic_id = {$topic['id']} AND p.poster_id = u.id " .
				                     "UNION SELECT {$fields}, 0, NULL FROM #__agora_posts AS p WHERE p.topic_id = {$topic['id']} AND p.poster_id = 0 " .
				                     "ORDER BY id ASC LIMIT 1");
				$post = $this->_db->loadAssoc();
			
				$email = $post['poster_email'];
				if ($email == 'null') {
					$email = $post['email'];
				}
				
				$catid = 0;
				if (isset(self::$new_forums_ids[$topic['forum_id']])) {
					$catid = self::$new_forums_ids[$topic['forum_id']];
				}

				/* map topic icons */
				switch ($topic['icon_topic']) {
				case 1: /* exclamation */
					$iconid = 1; /* exclamation */
					break;
				case 2: /* bulb */
					$iconid = 3; /* mark */
					break;
				case 3: /* ladybird */
					$iconid = 4; /* heart / love */
					break;
				case 4: /* crossed tools */
					$iconid = 1; /* exclamation */
					break;
				case 5: /* man (?) */
					$iconid = 0; /* default (document) */
					break;
				case 6: /* red cross */
					$iconid = 6; /* not amused / shock */
					break;
				case 7: /* smile */
					$iconid = 7; /* smile */
					break;
				case 8: /* document */
					$iconid = 0; /* default (document) */
					break;
				case 9: /* green tick */
					$iconid = 5; /* laugh / grin */
					break;
				case 10: /* note */
					$iconid = 2; /* question */
					break;
				case 0: /* default ("i") */
				default:
					$iconid = 0; /* default (document) */
				}
								
				unset($data);
				
				// Data array
				$data = array(
					'id' => $topic['id'],
					'parent' => 0,
					'thread' => $topic['id'],
					'catid' => $catid,
					'name' => $post['poster'],
					'userid' => $post['jos_id'],
					'email' => $email,
					'subject' => $topic['subject'],
					'time' => $post['posted'],
					'ip' => $post['poster_ip'],
					'topic_emoticon' => $iconid,
					'locked' => $topic['closed'],
					'ordering' => $topic['sticky'], /* yes, this is correct */
					'hits' => $topic['num_views'],
					'moved' => $topic['moved_to'],
					'modified_by' => $post['edited_by'],
					'modified_time' => $post['edited']
				);

				$id = self::_insertData('kunena_messages', $data);
				/* this is the first message of this topic, so we must store it's id */
				$this->firstPost[$topic['id']] = $post['id'];
				
				unset($data);
				
				// Data array
				$data = array(
					'mesid' => $topic['id'],
					'message' => $post['message']
				);

				self::_insertData('kunena_messages_text', $data);
			}
			
			$offset += QUERY_LIMIT;
			
		} while ($have_data);
	}
	
	/* Import posts from #__agora_posts with help of #__agora_posts and #__agora_users
	 * to #__kunena_messages and #__kunena_messages_text.
	 * Suppress already imported posts. (first of each topic)
	 */
	protected function _importAgoraPosts() {
		$offset = 0;
		
		$parents = array();
		
		do {
			$have_data = false;

		/* Because there could be anonymous posts this query doesn't work. We need a more advanced one.
			$fields = "p.id, p.poster, p.poster_id, p.poster_ip, p.poster_email, p.message, p.posted, p.edited, p.edited_by, p.topic_id, t.subject, t.closed, t.sticky, t.moved_to, t.forum_id, t.num_views, u.jos_id, u.email";
			$this->_db->setQuery("SELECT {$fields} FROM #__agora_posts AS p, #__agora_topics AS t, #__agora_users AS u WHERE p.topic_id = t.id AND p.poster_id = u.id ORDER BY p.id LIMIT {$offset}, " . QUERY_LIMIT);
		 */
			$fields = "p.id, p.poster, p.poster_id, p.poster_ip, p.poster_email, p.message, p.posted, p.edited, p.edited_by, p.topic_id, t.subject, t.closed, t.sticky, t.moved_to, t.forum_id, t.num_views";
			$this->_db->setQuery("SELECT {$fields}, u.jos_id, u.email FROM #__agora_posts AS p, #__agora_topics AS t, #__agora_users AS u WHERE p.topic_id = t.id AND p.poster_id = u.id " .
			                     "UNION SELECT {$fields}, 0, NULL FROM #__agora_posts AS p, #__agora_topics AS t WHERE p.topic_id = t.id AND p.poster_id = 0 " .
			                     "ORDER BY id ASC LIMIT {$offset}, " . QUERY_LIMIT);
			$ob_list = $this->_db->loadAssocList();

			if (!empty($ob_list)) {
				$have_data = true;
			} else {
				break;
			}

			foreach ($ob_list as $ob) {
				$email = $ob['poster_email'];
				if ($email == 'null') {
					$email = $ob['email'];
				}
				
				$catid = 0;
				if (isset(self::$new_forums_ids[$ob['forum_id']])) {
					$catid = self::$new_forums_ids[$ob['forum_id']];
				}
				
				if (!isset($parents[$ob['topic_id']])) {
					$parent = $ob['topic_id'];
				} else {
					$parent = $parents[$ob['topic_id']];
				}
				
				if ($this->firstPost[$ob['topic_id']] != $ob['id']) {
					unset($data);
				
					// Data array
					$data = array(
						'parent' => $parent,
						'thread' => $ob['topic_id'],
						'catid' => $catid,
						'name' => $ob['poster'],
						'userid' => $ob['jos_id'],
						'email' => $email,
						'subject' => $ob['subject'],
						'time' => $ob['posted'],
						'ip' => $ob['poster_ip'],
						'locked' => $ob['closed'],
						'ordering' => $ob['sticky'],
						'hits' => $ob['num_views'],
						'moved' => $ob['moved_to'],
						'modified_by' => $ob['edited_by'],
						'modified_time' => $ob['edited']
					);

					$id = self::_insertData('kunena_messages', $data);
					$parents[$ob['topic_id']] = $id;
				
					unset($data);
				
					// Data array
					$data = array(
						'mesid' => $id,
						'message' => $ob['message']
					);

					self::_insertData('kunena_messages_text', $data);
				}
			}
			
			$offset += QUERY_LIMIT;
			
		} while ($have_data);
	}
	
	/* Import polls from #__agora_polls to #__kunena_polls, #__kunena_polls_options, #__kunena_polls_users. */
	protected function _importAgoraPolls() {
		/* Hoffi: Because we don't have polls we don't need to import them ;) - sorry to all having polls */
	}

	/* Update id of last post for each category/forum in #__kunena_categories with help of #__kunena_messages. */
	protected function _updateLastMessageIDofCats() {
		$this->_db->setQuery("SELECT id FROM #__kunena_categories");
		$catids = $this->_db->loadResultArray();
		
		if (!empty($catids)) {
			foreach ($catids as $catid) {
				$this->_db->setQuery("SELECT id FROM #__kunena_messages WHERE catid = {$catid} ORDER BY id DESC LIMIT 1");
				$id_last_msg = $this->_db->loadResult();
				
				if (!empty($id_last_msg)) {
					$this->_db->setQuery("UPDATE #__kunena_categories SET id_last_msg = '{$id_last_msg}' WHERE id = {$catid}");
					$this->_db->query();
				}
			}
		}
	}

	/* Import subscriptions from #__agora_subscriptions with help of #__agora_users 
	 * to #__kunena_subscriptions or #__kunena_subscriptions_categories.
	 */	
	protected function _importAgoraSubscriptions() {
	/* Hoffi: This doesn't work! Also we must distinguish between category and topic subscriptions and get right user id. So we go the long way...
		$this->_db->setQuery("INSERT INTO #__kunena_subscriptions (userid, thread) SELECT user_id, topic_id FROM  #__agora_subscriptions");
		$this->_db->query();
	 */

		$offset = 0;

		do {
			$have_data = false;

			$this->_db->setQuery("SELECT u.jos_id, s.topic_id, s.forum_id FROM #__agora_subscriptions AS s INNER JOIN #__agora_users AS u ON u.id = s.user_id ORDER BY s.forum_id ASC LIMIT {$offset}, " . QUERY_LIMIT);
			$ob_list = $this->_db->loadAssocList();

			if (!empty($ob_list)) {
				$have_data = true;
			} else {
				break; /*do...while*/
			}

			foreach ($ob_list as $sub) {
				/* jos_id, topic_id, forum_id --> thread, userid / catid, userid */

				if ($sub['forum_id'] != 0) {
					/* forum subscription */
					if (isset(self::$new_forums_ids[$sub['forum_id']])) {
						$id = self::$new_forums_ids[$sub['forum_id']];
						unset($data);
						$data = array(
							'catid'  => $id,
							'userid' => $sub['jos_id']
						);

						self::_insertData('kunena_subscriptions_categories', $data);
					}
				} elseif ($sub['topic_id'] != 0) {
					/* topic subscription */
					unset($data);
					$data = array(
						'thread' => $sub['topic_id'],
						'userid' => $sub['jos_id']
					);

					self::_insertData('kunena_subscriptions', $data);
				}
			}

			$offset += QUERY_LIMIT;
		} while ($have_data);
	}

	/* Compare smiley lists and copy all smileys not present in Kunena.
	 * Import from #__agora_smilies to #_kunena_smileys
	 */
	protected function _importAgoraSmilies() {

		$this->_db->setQuery("SELECT * FROM #__agora_smilies ORDER BY id");
		$ob_list = $this->_db->loadAssocList();
		
		if (empty($ob_list)) {
			return;
		}
		
		$dir_ = dirname(dirname(dirname(dirname(dirname(__FILE__)))));
		$src_ = $dir_.DS.'components'.DS.'com_agora'.DS.'img'.DS.'smilies'.DS;
		$dst_ = $dir_.DS.'components'.DS.'com_kunena'.DS.'template'.DS.'default'.DS.'images'.DS.'emoticons'.DS;

		foreach ($ob_list as $ob) {
			/* check if this smiley is already defined */
			$this->_db->setQuery("SELECT id FROM #__kunena_smileys WHERE code LIKE '{$ob['text']}' LIMIT 1");
			$found = $this->_db->loadResult();

			if (empty($found)) {
				unset($data);
				$data = array(
					'code' => $ob['text'],
					'location' => $ob['image'],
					'emoticonbar' => '1'
				);

				/* copy the file (don't overwrite files with same name) */
				if ((IMPORT_SMILEYS_MODE >= 2) or (!JFile::exists($dst_.$data['location']))) {
					if (JFile::exists($src_.$ob['image'])) {
						JFile::copy($src_.$ob['image'], $dst_.$data['location']);
					} else {
						continue; /*foreach*/ /* don't add to db */
					}
				}

				self::_insertData('kunena_smileys', $data);
			}
		}
	}

	/* Converts all references to attached files.
	 * Note: Files storage location is not changed. So if you remove Agora attachments get lost!
	 */
	protected function _convertAttachement($message) {
		// Url -> [file name=changelog.txt size=9023]http://127.0.0.1/mediaconweb.it/images/fbfiles/files/changelog.txt[/file]
		$pattern = array('#\[file name=(.*?) size=[0-9]*\](.*?)\[/file\]#i');
		$replace = array('[url=$2]$1[/url]');

		return preg_replace($pattern, $replace, $message);
	}


	/* Helper functions */

	protected function _insertData($table, $data) {
		// Put together the query
		$names = array();
		$vars = array();

		foreach($data as $name => $var) {
			if ($var != '') {
				$names[] = $name;
				$var == 'null' ? $vars[] = $var : $vars[] = '\''.addslashes(stripslashes(self::_convertHTML($var))).'\'';
			}
		}
		
		$query = 'INSERT INTO #__'.$table.' ('.implode(',', $names).') VALUES ('.implode(',', $vars).')';
		$this->_db->setQuery($query);
		$this->_db->query();
		
		if ($this->_db->getErrorNum()) {
			throw new Exception(__METHOD__.' failed. ('.$this->_db->getErrorMsg().')');
		}
		
		return $this->_db->insertID();
	}
	
	protected function _convertHTML($message) {		
		$pattern = array(
		'/&gt;/i',
		'/&lt;/i',
		'/&amp;/i',
		'/&quot;/i',
		'/&#039;/i'
		);

		$replace = array(
		'>',
		'<',
		'&',
		'"',
		"'"
		);
		
		return preg_replace($pattern, $replace, $message);		
	}
	
	// Check info
	public function &getInfo() {
        $info = array();
		
        // Get installed version
        $info['version_installed'] = $this->getInstalledVersion();
       
        // Get latest version
        $info['version_latest'] = '?.?.?';
        $content = self::getRemoteData('http://www.joomace.net/index.php?option=com_aceversions&view=xml&format=xml&catid=6');
       
        if (strstr($content, '<?xml version="1.0" encoding="UTF-8" ?>')) {
			$xml =& JFactory::getXMLparser('Simple');
			$xml->loadString($content);
			
			$category = $xml->document->getElementByPath('category');
			
			foreach ($category->children() as $extension) {
				$option = $extension->attributes('option');
				if ($option == 'com_kunenaconverters') {
					$info['version_latest'] = trim($extension->attributes('version'));
					break;
				}
			}
		}
       
        // Set the version status
        $info['version_status'] = version_compare($info['version_installed'], $info['version_latest']);
        $info['version_enabled'] = 1;
       
        return $info;
    }

	protected function &getInstalledVersion()  {
        static $version;

        if (!isset($version)) {
            $xml = JFactory::getXMLParser('Simple');

            $xmlFile = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_kunenaconverters'.DS.'kunenaconverters.xml';

             if (JFile::exists($xmlFile)) {
                if ($xml->loadFile($xmlFile)) {
                    $root =& $xml->document;
                    $element =& $root->getElementByPath('version');
                    $version = $element ? $element->data() : '';
                }
            }
			
        }

        return $version;
    }
	
	protected function getRemoteData($url) {
		$useragent = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.3) Gecko/20090824 Firefox/3.5.3 (.NET CLR 3.5.30729)";
		$data = false;

		// cURL
		if (extension_loaded('curl')) {
			// Init cURL
			$ch = @curl_init();
			
			// Set options
			@curl_setopt($ch, CURLOPT_URL, $url);
			@curl_setopt($ch, CURLOPT_HEADER, 0);
			@curl_setopt($ch, CURLOPT_FAILONERROR, 1);
			@curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			@curl_setopt($ch, CURLOPT_USERAGENT, $useragent);
			
			// Set timeout
			@curl_setopt($ch, CURLOPT_TIMEOUT, 5);
			
			// Grab data
			$data = @curl_exec($ch);
			
			// Clean up
			@curl_close($ch);
			
			// Return data
			if ($data !== false) {
				return $data;
			}
		}

		// fsockopen
		if (function_exists('fsockopen')) {
			$errno = 0;
			$errstr = '';
			
			$url_info = parse_url($url);
			if($url_info['host'] == 'localhost')  {
				$url_info['host'] = '127.0.0.1';
			}

			// Set timeout
			$fsock = @fsockopen($url_info['scheme'].'://'.$url_info['host'], 80, $errno, $errstr, 5);
		
			if ($fsock) {
				@fputs($fsock, 'GET '.$url_info['path'].(!empty($url_info['query']) ? '?'.$url_info['query'] : '').' HTTP/1.1'."\r\n");
				@fputs($fsock, 'HOST: '.$url_info['host']."\r\n");
				@fputs($fsock, "User-Agent: ".$useragent."\n");
				@fputs($fsock, 'Connection: close'."\r\n\r\n");
		
				// Set timeout
				@stream_set_blocking($fsock, 1);
				@stream_set_timeout($fsock, 5);
				
				$data = '';
				$passed_header = false;
				while (!@feof($fsock)) {
					if ($passed_header) {
						$data .= @fread($fsock, 1024);
					} else {
						if (@fgets($fsock, 1024) == "\r\n")
							$passed_header = true;
					}
				}
				
				// Clean up
				@fclose($fsock);
				
				// Return data
				if ($data !== false) {
					return $data;
				}
			}
		}

		// fopen
		if (function_exists('fopen') && ini_get('allow_url_fopen')) {
			// Set timeout
			if (ini_get('default_socket_timeout') < 5) {
				ini_set('default_socket_timeout', 5);
			}
			@stream_set_blocking($handle, 1);
			@stream_set_timeout($handle, 5);
			@ini_set('user_agent',$useragent);
			
			$url = str_replace('://localhost', '://127.0.0.1', $url);
			
			$handle = @fopen ($url, 'r');
			
			if ($handle) {
				$data = '';
				while (!feof($handle)) {
					$data .= @fread($handle, 8192);
				}
				
				// Clean up
				@fclose($handle);
			
				// Return data
				if ($data !== false) {
					return $data;
				}
			}
		}
		
		// file_get_contents
		if(function_exists('file_get_contents') && ini_get('allow_url_fopen')) {
			$url = str_replace('://localhost', '://127.0.0.1', $url);
			@ini_set('user_agent',$useragent);
			$data = @file_get_contents($url);
			
			// Return data
			if ($data !== false) {
				return $data;
			}
		}
		
		return $data;
	}
}
